"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cmd_ts_1 = require("cmd-ts");
const fs_1 = require("cmd-ts/batteries/fs");
const path_1 = __importDefault(require("path"));
const StringList = {
    from(input) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Set(input.split(",").map((s) => s.trim()));
        });
    },
};
const srcDir = (0, cmd_ts_1.positional)({
    type: fs_1.Directory,
    displayName: "Source dir",
    description: "Directory of the targeted application",
});
const outDir = (0, cmd_ts_1.positional)({
    type: {
        from: (s) => __awaiter(void 0, void 0, void 0, function* () {
            if (s.startsWith("/"))
                return s;
            return path_1.default.join(process.cwd(), s);
        })
    },
    displayName: "Out dir",
    description: "Output directory",
});
const include = (0, cmd_ts_1.option)({
    type: StringList,
    long: "include",
    short: "i",
    defaultValue: () => undefined,
    description: "Comma separated list of node-module to include",
});
const exclude = (0, cmd_ts_1.option)({
    type: StringList,
    long: "exclude",
    short: "e",
    defaultValue: () => undefined,
    description: "Comma separated list of node-module to exclude",
});
const wrapperName = (0, cmd_ts_1.option)({
    type: cmd_ts_1.string,
    long: "name",
    short: "n",
    description: 'Wrapper name, (default "ø")',
    defaultValue: () => "ø",
});
const deletee = (0, cmd_ts_1.flag)({
    long: "delete",
    short: "d",
    defaultValue: () => false,
    description: "Delete the output directory before running.",
});
const moduleName = (0, cmd_ts_1.option)({
    type: {
        from: (input) => __awaiter(void 0, void 0, void 0, function* () {
            const s = input.toLocaleLowerCase();
            const validModes = ["classic", "minimal"];
            if (validModes.includes(s)) {
                return s;
            }
            throw new Error(`Invalid module name "${input}"`);
        })
    },
    long: 'module',
    short: 'j',
    defaultValue: () => 'classic',
    description: "Which module to use ( classic, minimal )"
});
const hookMode = (0, cmd_ts_1.option)({
    type: {
        from: (input) => __awaiter(void 0, void 0, void 0, function* () {
            const s = input.toLocaleLowerCase();
            const validModes = ["inline", "require"];
            if (validModes.includes(s)) {
                return s;
            }
            throw new Error(`Invalid hook mode "${input}"`);
        })
    },
    long: 'mode',
    short: 'm',
    defaultValue: () => 'require',
    description: "Which mode to use to inject pp-finder"
});
exports.default = {
    delete: deletee,
    wrapperName,
    exclude,
    include,
    outDir,
    srcDir,
    hookMode,
    module: moduleName
};
