"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCompiler = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const transformer_1 = require("./transformer");
function createCompiler({ minimal, wrapperName, inline, }) {
    const module = minimal ? "minimal" : "classic";
    const modulePath = path_1.default.join(path_1.default.dirname(__filename), "module", `index.${module}.js`);
    const moduleContent = fs_1.default.readFileSync(modulePath, "utf8");
    function compile(filename, source) {
        const tree = typescript_1.default.createSourceFile("", source, typescript_1.default.ScriptTarget.ESNext, true, typescript_1.default.ScriptKind.JS);
        const transformer = (context) => {
            const utils = {
                wrapperName: wrapperName,
                visit(node) {
                    return typescript_1.default.visitNode(node, visit);
                },
                createWrapperCall(name, target, params) {
                    const pos = typescript_1.default.getLineAndCharacterOfPosition(tree, target.getStart());
                    const posArray = typescript_1.default.factory.createArrayLiteralExpression([
                        typescript_1.default.factory.createNumericLiteral(pos.line + 1),
                        typescript_1.default.factory.createNumericLiteral(pos.character + 1),
                    ]);
                    return typescript_1.default.factory.createCallExpression(typescript_1.default.factory.createIdentifier(`${this.wrapperName}.${name}`), undefined, [...params, posArray]);
                },
            };
            const visit = (node) => {
                const ranges = typescript_1.default.getLeadingCommentRanges(source, node.getFullStart());
                let skipTransformer = false;
                if (ranges === null || ranges === void 0 ? void 0 : ranges.length) {
                    const comments = ranges.map((r) => source.slice(r.pos, r.end));
                    for (const range of ranges) {
                        const comment = source.slice(range.pos, range.end);
                        if (comment == "/* harmony export */" ||
                            comment == "/* harmony import */" ||
                            comment == "/* harmony default export */") {
                            return node;
                        }
                    }
                }
                if (!skipTransformer)
                    for (const transformer of transformer_1.transformers) {
                        const newNode = transformer(node, utils);
                        if (newNode) {
                            return newNode;
                        }
                    }
                return typescript_1.default.visitEachChild(node, visit, context);
            };
            return (node) => typescript_1.default.visitNode(node, visit);
        };
        const printer = typescript_1.default.createPrinter();
        let prefix = "";
        if (inline) {
            prefix = `const ${wrapperName} = ((() => { const exports = {}; ${moduleContent} ; exports.ppFinder = ppFinder; return exports; })()).ppFinder(${JSON.stringify(filename)});\n`;
        }
        else {
            prefix = `const ${wrapperName} = require("pp-finder").ppFinder(${JSON.stringify(filename)});\n`;
        }
        const transformedTree = typescript_1.default.transform(tree, [transformer]);
        const code = transformedTree.transformed.map((n) => printer.printFile(n));
        return prefix + code.join("\n");
    }
    return {
        compile,
    };
}
exports.createCompiler = createCompiler;
